################################
# MDP Microbes Application Study
#
# This script sets up the data and runs the MDP estimations
# Code written by Michelle LaMar


####################################################
#   Data loading and manipulations
#
# Data provided by WestEd and Redhill Studios
#

# Read in the play data from the log files.  This includes play on levels 5-10 (the first 4 being largely tutorial).
playRecFile = "microbePlayData.txt"
microbeDat = read.table(playRecFile)
colnames(microbeDat) = c("studID", "level", "numMito", "numChloro", "action", "result")

# Returning to a level is not a problem until they have won it, then they seem to be messing around
# So we create a subset of the play records that exclude actions on a level once the student has won it
# for the first time.

studList = unique(microbeDat$studID)
wonLev = matrix(0, nrow = length(studList), ncol = 6)
excludeRows = vector()
for(r in 1:nrow(microbeDat)) {
	studIndex = which(studList == microbeDat$studID[r])
	levIndex = microbeDat$level[r] - 4
	if(wonLev[studIndex, levIndex] == 1) {
		excludeRows = c(excludeRows, r)
	} else {
		if(microbeDat$result[r] == "Success")
			wonLev[studIndex, levIndex] = 1
	}
}

microbeDat_tilWin = microbeDat[-excludeRows,]

# Also take out everything over 10 mitochondria or 10 chloroplasts as that violates the size of
# our state space:
length(unique(subset(microbeDat, numMito > 10 | numChloro > 10)$studID))/length(unique(microbeDat$studID))
nrow(subset(microbeDat, numMito > 10 | numChloro > 10))/nrow(microbeDat)

microbeDat_tilWin2 = subset(microbeDat_tilWin, numMito <= 10 & numChloro <=10)
# And on the full data
microbeDat2 = subset(microbeDat, numMito <= 10 & numChloro <=10)

playRecFile_tw = "microbePlayData_tw.txt"
playRecFile_tw2 = "microbePlayData_tw2.txt"
playRecFile2 = "microbePlayData2.txt"

write.table(microbeDat_tilWin, playRecFile_tw , row.names = F, col.names = F, quote = F)
write.table(microbeDat_tilWin2, playRecFile_tw2 , row.names = F, col.names = F, quote = F)
write.table(microbeDat2, playRecFile2, row.names = F, col.names = F, quote = F)

# Get student play statistics:
studRecDat = aggregate(cbind(numMito, numChloro) ~ studID, data = microbeDat_tilWin2, max)
colnames(studRecDat) = c("studID", "Max_Mito", "Max_Chloro")
srd2 = aggregate(studID > 0 ~ studID, data = microbeDat_tilWin2, sum)
colnames(srd2) = c("studID", "Rec_Length")
srd3 = aggregate(result == "Success" ~ studID, data = microbeDat_tilWin2, sum)
colnames(srd3) = c("studID", "Num_Win")
levTab = with(microbeDat_tilWin2, table(studID, level))
srd4 = data.frame(studID = as.numeric(rownames(levTab)), LevPlayed = rowSums(levTab > 0))

studRecDat = merge(studRecDat, srd2)
studRecDat = merge(studRecDat, srd3)
studRecDat = merge(studRecDat, srd4)
studRecDat$LevQuit = studRecDat$LevPlayed - studRecDat$Num_Win
studRecDat$PropQuit = studRecDat$LevQuit/studRecDat$LevPlayed 

studRecDatTrimmed = studRecDat


# Create a data set that removes all students who never bought an organelle, they seem confused by the UI

noBuys = subset(studRecDatTrimmed, Max_Mito == 0 & Max_Chloro == 0)$studID
microbeDat_tw2_wb = subset(microbeDat_tilWin2, !(studID %in% noBuys))
playRecFile_wb = "microbePlayRec_withBuys.txt"
write.table(microbeDat_tw2_wb, playRecFile_wb , row.names = F, col.names = F, quote = F)

#######################################################################3
#      MDP Models
#
# We are estimating two models
# Common model arguments:

# Reward parameters are: (R_win, ?, )
rewArg = "--rewardValues \"(1, 0, -0.2, -0.1, -0.1, 0)\""

# Transition prameters control the coefficients in the logistic regression for probability of winning after
# an Enter-Tank action
transParamArg = "--transParams \"(-5, 1, 1, 0, 1, 2)\""

# Which game levels we are using from the data
levArg = paste("--levelRange ", 5, ":", 10, sep = "")

# Model 1 -- fixed reward parameters
mod1_REstArg = ""   # Estimating nothing

# Model 2 -- 2 estimated reward parameters 
mod2_REstArg = "--estRBuy --estRLose"

################################################
# Estimation
#

# Run the two models on the data (excluding the post-level-win actions ???)

# Runs on trimmed data (playRecFile_tw2), using R_win = 5
# Then we will run everything again on the full data

fullPlayDatFile = "microbePlayData_tw2.txt"
#fullPlayDatFile = "microbePlayData2.txt"

# Model 1
estModFile = "mic_modEst_mod1.out"  # This will contain the model parameter estimates (mu & sigma)
estPerFileMle = "mic_perEst_mod1.mle"  # This will contain the mle person-level beta estimates

estCmd = paste("MicrobeMDP.exe --estPersonParams  --persEstMLE", levArg, mod1_REstArg, "--personParamFile", estPerFileMle,
		 "--modelParamFile", estModFile, "--dataInputFile", fullPlayDatFile, rewArg, transParamArg)

system(estCmd)

# Model 2
estModFile = "mic_modEst_mod2.out"  # This will contain the model parameter estimates (mu & sigma & rewards)
estPerFileMle = "mic_perEst_mod2.mle"  # This will contain the mle person-level beta estimates

estCmd = paste("MicrobeMDP.exe --estPersonParams  --persEstMLE", levArg, mod2_REstArg, "--personParamFile", estPerFileMle,
		 "--modelParamFile", estModFile, "--dataInputFile", fullPlayDatFile, rewArg, transParamArg)

system(estCmd)


# Now we re-estimate the Model 2 model parameters using only the student who bought organelles, this is called Model 2' (or 2p for file names)
wbPlayDatFile = "microbePlayRec_withBuys.txt"
estModFile = "mic_modEst_mod2p.out"  # This will contain the model parameter estimates based on with-buy students only
estPerFileMle = "mic_perEst_mod2p_wb.mle"  # Beta estimates for with-buy students only

estCmd = paste("MicrobeMDP.exe --estPersonParams  --persEstMLE", levArg, mod2_REstArg, "--personParamFile", estPerFileMle,
		 "--modelParamFile", estModFile, "--dataInputFile", wbPlayDatFile, rewArg, transParamArg)

system(estCmd)

# And then re-run the full data file with the reward parameters fixed to those estimated by the with-buys students.  
# This is accomplished by passing in the same model param file as was output estimating the with-buys data.  These
# values will then be read from the file rather than estimated

estModFile = "mic_modEst_mod2p.out"  # The same file as from above
estPerFileMle = "mic_perEst_mod2p.mle"  # This will contain the full mle person-level beta estimates for model 2'

estCmd = paste("MicrobeMDP.exe --estPersonParams  --persEstMLE", levArg, mod2_REstArg, "--personParamFile", estPerFileMle,
		 "--modelParamFile", estModFile, "--dataInputFile", fullPlayDatFile, rewArg, transParamArg)

system(estCmd)

