################################
# Sim Study 1
#
# Looking at basic parameter recovery of theta and the theta distribution. 
# (Originally called study 's0d')
# This code analyzes the estimation results from the simulation study.
# Note that code for generation and estimation can be found in SimStudy1_genEstRecs.R
#
# Code written by Michelle LaMar

# Start in the directory above the directories with the play records.
simDir =  getwd()
setwd(simDir)

############################################
# Utility function
#

rmse = function(est, true) {
	return(sqrt( mean( (est - true)^2)))
}
############################################

# Number of simulated students
N = 200
# Number of run repetitions
nReps = 50

# Read the generating true person parameter file:
trueThetaFile = "S0d_trueLogTheta.txt"
trueLogTheta = read.table(trueThetaFile)$V2
trueTheta = exp(trueLogTheta)

# run numbers as strings with leading zeros (for filenames)
runStr = formatC(1:nReps, width = nchar(as.character(nReps)), flag = "0")

# The boardnames were changed in the paper for clarity.  Mapping is:
# PlusPlusBoard -> Tiny Cross
# ThinCross -> Big Cross
# Cross_1100 -> Big L
# Diamond -> Diamond
boardList = c("PlusPlusBoard", "ThinCross", "Cross_1100", "Diamond")
dirList = paste(boardList, "runs", sep = "_")
dirList = c(dirList, "Four-task")
boardNames = c(boardList, "Four-task")
boardFiles = paste("../boards/", boardList,".txt", sep = "")

# Collect up all the data into a couple of data frames
# We will track the theta errors for both the lognormal and normal distributions

studStr = formatC(1:N, width = nchar(as.character(N)), flag = "0")

# We track the statistics in the log-transformed distribution as that
# is a normal distribution, and so comparable to more familiar error statistics
# Note that estimates are reported in log-transformed form.

S0d_logThetaEstList = vector(mode = "list")
S0d_logPopEstList = vector(mode = "list")
meanEstLogThetaMat = matrix(NA, ncol = N, nrow = length(dirList))
sdEstLogThetaMat = matrix(NA, ncol = N, nrow = length(dirList))
perStudLogBiasMat = matrix(NA, ncol = N, nrow = length(dirList))
perStudLogRmseMat = matrix(NA, ncol = N, nrow = length(dirList))

startReps = 1
endReps = 50

for(d in 1:length(dirList)) {
	setwd(simDir)
	setwd(dirList[d])
	# Create the filenames for the beta (theta) and model (mu and sigma) estimation output files:
	thetaEstFiles = paste("S0d_thetaEst_r", runStr[startReps:endReps], ".map", sep = "")
	modEstFiles = paste("S0d_modEst_r", runStr[startReps:endReps], ".mle", sep = "")

	# Create the beta estimate matrices pre-labels with run numbers and student ids
	S0d_logThetaEstList[[d]] = matrix(NA, nrow = length(thetaEstFiles), ncol = N, dimnames = list(paste("run",runStr[1:length(thetaEstFiles)], sep="_"), paste("stud", studStr, sep="_")))

	# Create the population distribution estimation matricies with rows as run numbers and 2 columns: mu and sigma
	S0d_logPopEstList[[d]] = matrix(NA, nrow = length(modEstFiles), ncol = 2, dimnames = list(paste("run",runStr[1:length(modEstFiles)], sep="_"), c("thetaMuHat", "thetaSigmaHat")))

	# Fill in the log beta estimates from the estimation files for each replication (run).  Calculate the beta estimates.
	run = 1
	for(thFile in thetaEstFiles) {
		S0d_logThetaEstList[[d]][run,] = read.table(thFile)$V2
		run = run + 1
	}

	# Fill in the model estimates for each run
	run = 1
	for(mFile in modEstFiles) {
		S0d_logPopEstList[[d]][run,] = read.table(mFile)$V2[1:2]
		run = run + 1
	}

	# Now get the actual population mean and sd:
	meanEstLogThetaMat[d,] = apply(S0d_logThetaEstList[[d]], 2, mean)
	sdEstLogThetaMat[d,] = apply(S0d_logThetaEstList[[d]], 2, sd)

	# Bias is the mean beta estimate minus the true beta:
	perStudLogBiasMat[d,] =  apply(S0d_logThetaEstList[[d]], 2, mean) - trueLogTheta

	# See RMSE function lower down in the code.
	for(s in 1:N) {
		perStudLogRmseMat[d,s] = rmse(S0d_logThetaEstList[[d]][,s], trueLogTheta[s])
	}

	setwd("..")
}

# Label the matrices
rownames(meanEstLogThetaMat) = boardNames
rownames(sdEstLogThetaMat) = boardNames
rownames(perStudLogBiasMat) = boardNames
rownames(perStudLogRmseMat ) = boardNames


# Aggregate Population Parameter estimates:
logPopMat = matrix(nrow = length(boardNames), ncol = 4)

for(b in 1:length(boardNames)) {
	logPopMat[b,c(1,3)] = apply(S0d_logPopEstList[[b]], 2, mean)
	logPopMat[b,c(2,4)] = apply(S0d_logPopEstList[[b]], 2, sd)

}
colnames(logPopMat) = c("Mean mu_theta", "SD mu_theta", "Mean sigma_theta", "SD sigma_theta")
rownames(logPopMat) = boardNames

# Population Parameters
# Table 4 in the paper:

logPopErrMat = matrix(nrow = length(boardNames), ncol = 4)

for(b in 1:length(boardNames)) {

	logPopErrMat[b,1] = mean(S0d_logPopEstList[[b]][,1]) - mean(trueLogTheta)
	logPopErrMat[b,2] = rmse(S0d_logPopEstList[[b]][,1],rep(mean(trueLogTheta),length(S0d_logPopEstList[[b]][,1])))

	logPopErrMat[b,3] = mean(S0d_logPopEstList[[b]][,2]) - sd(trueLogTheta)
	logPopErrMat[b,4] = rmse(S0d_logPopEstList[[b]][,2],rep(sd(trueLogTheta),length(S0d_logPopEstList[[b]][,1])))

}
# Label everything nicely
colnames(logPopErrMat) = c("Bias mu_theta", "RMSE mu_theta", "Bias sigma_theta", "RMSE sigma_theta")
rownames(logPopErrMat) = boardNames

# Total bias and RMSE for student level theta
# Table 5 in the paper:

logThetaErrMat = matrix(nrow = length(boardNames), ncol = 4)

for(b in 1:length(boardNames)) {

	perStudLogErr = apply(S0d_logThetaEstList[[b]], 1, function(x) { x - trueLogTheta})
	logThetaErrMat[b,1] = mean(perStudLogErr)
	logThetaErrMat[b,2] = sqrt(mean(perStudLogErr^2))
	logThetaErrMat[b,3] = sd(perStudLogErr)/sqrt(length(perStudLogErr))
	logThetaErrMat[b,4] = sqrt(sd(perStudLogErr^2)/sqrt(length(perStudLogErr)))
}

rownames(logThetaErrMat) = boardNames
colnames(logThetaErrMat) = c("Bias_theta", "RMSE_theta", "Bias_se", "RMSE_se")

### FOR PAPER ** Log hat(beta) by log(true-beta) #################
# require("plotmath")
par(mar = c(5,5,3,2), cex.lab = 1.3, cex.axis = 1.2)
plot(meanEstLogThetaMat[5,] ~ trueLogTheta, # main = "Recovery of beta on Four-task Assessment",
ylim = range(trueLogTheta), xlab = expression(paste("True log( ", beta,")")), 
			ylab = expression(paste("Mean log( ",hat(beta),")"))  )
abline(0,1, col = "red")


