################################
# Sim Study 1
#
# Looking at basic parameter recovery of theta and the theta distribution.
# (Originally called study 's0d')
# This code generates the simulated students and the simulated game play records.
# Then the beta parameters (here called 'theta') are estimated based on the play records.
# Note that code for analysis of the resuts can be found in SimStudy1_analysis.R
#
# Code written by Michelle LaMar

# Start where you want the records to be generated
simDir = getwd()
setwd(simDir)

# Set overall parameters.
# Number of simulated students
N = 200
# Number of run repetitions
nReps = 50

# These are the parameters in the generating of the log-normal distribution
ThetaMu = 0
ThetaSigma = 0.75

# The discount parameter
genDiscount = 1.0
estDiscount = 1.0

# Set this to false once the records have been generated to prevent
# writing over the generated files.
genPlayRecs = true  

# Create the person file (generated true theta):

trueLogTheta = round(rnorm(N, ThetaMu, ThetaSigma), 4)
trueTheta = exp(trueLogTheta)

trueThetaFile = "S0d_trueLogTheta.txt"
write.table(cbind(1:N, trueLogTheta), trueThetaFile, row.names = F, col.names = F)

# The string version of the run numbers (to put leading zeros in)
runStr = formatC(1:nReps, width = nchar(as.character(nReps)), flag = "0")

# The game boards
boardList = c("PlusPlusBoard", "ThinCross", "Cross_1100", "Diamond")
dirList = paste(boardList, "runs", sep = "_")
boardFiles = paste("../../boards/", boardList,".txt", sep = "")

condArgs = "--allowReset 1 --rewardValues \"(5, 1, -0.1, -1.0)\" "

# Loop over all of the game boards nReps times.  
# For each board and each repetition, generate play records (one play per simulated player)
# Then estimate the population and person parameters for that repetition and that game board
# (these are the single-game estimates -- combined estimates come later)
for(run in 1:nReps) {
for(b in 1:length(dirList)) {
	setwd(simDir)
	dir = dirList[b]
	if(!file.exists(dir)) {
		dir.create(dir)
	} 
	setwd(dir)

	# Generate the data
	dataPlayFile = paste("S0d_playRecs_r", runStr[run],".out", sep = "")

	if(genPlayRecs) {
		print(paste("Starting Data Gen for run", run, "for", boardFiles[b]))
		timestamp()

		genCmd = paste("../../PegBoardGameSim_ln.exe --genData --recActionChoice --boardFile", boardFiles[b],
			"--dataOutputFile", dataPlayFile, " --personParamFile", 
			paste("../",trueThetaFile,sep = ""), "--discount", genDiscount, condArgs)
		system(genCmd)
	}

	# Fit the model to the just generated data (estimation based on a single task)
	print(paste("Starting Estimation for run", run, boardFiles[b]) )
	timestamp()

	thetaOutFile = paste("S0d_thetaEst_r", runStr[run], ".map", sep = "")
	modOutFile = paste("S0d_modEst_r", runStr[run], ".mle", sep = "")
	estCmd = paste("../../PegBoardGameSim_ln.exe --estPersonParams --boardFile", boardFiles[b],
		"--dataInputFile", dataPlayFile, "--personParamFile", thetaOutFile, "--modelParamFile", modOutFile,
		"--discount", estDiscount, condArgs)
	system(estCmd)
	timestamp()
	print("Finished Estimation")

} # End board loop
} # End run loop

# Now the aggregated, multi-task estimations (full assessment):
if(!file.exists("Four-task")) {
	dir.create("Four-task")
} 
for(run in 1:nReps) {
	setwd(simDir)

	# These arguments include play records for all four boards for this run
	dataPlayFiles = paste("--dataInputFile ", dirList, "/S0d_playRecs_r", runStr[run],".out", sep = "", collapse = " ")
	boardArgs = paste("--boardFile ", boardFiles, sep = "", collapse = " ")

	# Fit the model
	print(paste("Starting Estimation for run", run) )
	timestamp()

	thetaOutFile = paste("Four-task/S0d_thetaEst_r", runStr[run], ".map", sep = "")
	modOutFile = paste("Four-task/S0d_modEst_r", runStr[run], ".mle", sep = "")
	estCmd = paste("PegBoardGameSim.exe --estPersonParams ", boardArgs,
		dataPlayFiles, "--personParamFile", thetaOutFile, "--modelParamFile", modOutFile,
		"--discount", estDiscount, condArgs)
	system(estCmd)
	timestamp()
	print("Finished Estimation")

} # End run loop