################################
# Sim Study 2
#
# Looking for ability to distinguish between population-level difference
# in ability and motivation. 
# (Originally called study 's4')
# This code generates data and estimates parameters for the simulation study.
# Note that code for analysis of results can be found in SimStudy2_analysis.R
#
# Code written by Michelle LaMar

simDir =  getwd()
setwd(simDir)

# Set overall parameters.  These do not change for this sim:
N = 200
nReps = 50

# We will cross two factors with two categories each:
# ability and motivation (Reward params), to give 4 populations:

popNames = c("High Ab : High Mot", "High Ab : Low Mot", "Low Ab : High Mot", "Low Ab : Low Mot")
popLabs = c("HAHM", "HALM", "LAHM", "LALM")

trueThetaFile_HA = "S4_trueLogTheta_HA.txt"
trueThetaFile_LA = "S4_trueLogTheta_LA.txt"

# These are the parameters in the of the log-normal distribution
# '_la' is low ability and '_ha' is the high ability 
ThetaMu_la = -0.5
ThetaSigma_la = 0.75
ThetaMu_ha = 0.5
ThetaSigma_ha = 0.75

# Set this to zero when thetas have been generated so we won't accidentally regen the thetas
genThetaVals = 1  

# Generate the theta values for high and low ability populations
# Note that we use the same values for high and low motivation play
if(genThetaVals) {
  trueLogTheta_HA = rnorm(N, ThetaMu_ha, ThetaSigma_ha)
  trueLogTheta_LA = rnorm(N, ThetaMu_la, ThetaSigma_la)

  # Write out the files
  write.table(cbind(1:N, trueLogTheta_HA), trueThetaFile_HA, row.names = F, col.names = F)
  write.table(cbind(1:N, trueLogTheta_LA), trueThetaFile_LA, row.names = F, col.names = F)
}

genPlayRecs = 1

startReps = 1
endReps = nReps

# Create string versions of the run numbers for the file names
runStr = formatC(1:nReps, width = nchar(as.character(nReps)), flag = "0")

# The peg-boards that we use:
boardList = c("PlusPlusBoard", "ThinCross", "Cross_1100", "Diamond")
dirList = paste(boardList, "runs", sep = "_")
boardFiles = paste("../../boards/", boardList,".txt", sep = "")

# Arguments for generating the play records.
#
# The motivation factor is modeled by changing the reward value for the
# move action.  "HM" -> High Motivation play has a smaller cost per move.
# Note that the reward value arguments are as "(winReward, pegPenalty, moveCost, resetCost)"
condHM = "--allowReset 1 --rewardValues \"(5, 1, -0.05, -1.0)\" "
condLM = "--allowReset 1 --rewardValues \"(5, 1, -0.75, -1.0)\" "

# Arguments for estimating the records.  Note that the --estMoveCost argument causes the moveCost parameter
# to be estimated, the -1 value passed in here is ignored.
estCondArgs = c("--estMoveCost --allowReset 1 --rewardValues \"(5, 1, -1, -1.0)\" ")

condArgs = c(condHM, condLM, condHM, condLM)
condTLT_files = c(trueThetaFile_HA, trueThetaFile_HA, trueThetaFile_LA, trueThetaFile_LA)

# First just generate the play records:
if(genPlayRecs) {
for(run in startReps:endReps) {
for(b in 1:length(boardList)) {
for(c in 1:length(condArgs)) { 

	setwd(simDir)
	gendir = paste(boardList[b],popLabs[c],sep = "_")

	if(!file.exists(gendir)) {
		dir.create(gendir)
	} 

	setwd(gendir)

	# Generate the data
	dataPlayFile = paste("S4_playRecs_r", runStr[run],".out", sep = "")


	print(paste("Starting Data Gen for run", run, "for board", boardList[b], "cond", popLabs[c]))
	timestamp()

	genCmd = paste("../../PegBoardGameSim_lnr.exe --genData --recActionChoice --boardFile", boardFiles[b],
		"--dataOutputFile", dataPlayFile, " --personParamFile", 
		paste("../",condTLT_files[c],sep = ""), condArgs[c])
	system(genCmd)

	}
	timestamp()

} # End cond loop
} # End board loop
} # End run loop

startReps = 1
endReps = nReps
# Now estimating the parameters:
for(run in startReps:endReps) {
for(c in 1:length(condArgs)) { 

	setwd(simDir)
	gendirs = paste(boardList,popLabs[c],sep = "_")
	estdir = paste("MultiTask",popLabs[c],"est",sep = "_")

	if(!file.exists(estdir)) {
		dir.create(estdir)
	} 
	setwd(estdir)

	# Fit the model
	print(paste("Starting Estimation for run",  run, "for condition", popLabs[c]) )
	timestamp()

	dataPlayFile = paste("S4_playRecs_r", runStr[run],".out", sep = "")
	thetaOutFile = paste("S4_thetaEst_r", runStr[run], ".map", sep = "")
	modOutFile = paste("S4_modEst_r", runStr[run], ".mle", sep = "")

	dataInputArgs = paste("--dataInputFile ../", gendirs, "/S4_playRecs_r", runStr[run],".out", sep = "", collapse = " ")
	boardArgs = paste("--boardFile ", boardFiles, sep = "", collapse = " ")

	sysCmd = paste("../../PegBoardGameSim.exe --estPersonParams ", boardArgs,
		dataInputArgs, "--personParamFile", thetaOutFile, "--modelParamFile", modOutFile,
		estCondArgs )

	system(sysCmd)
	timestamp()
	print("Finished Estimation")

} # End Est loop
} # End run loop

